import 'package:models_weebi/common.dart';
import 'package:models_weebi/src/models/article_calibre.dart';
import 'package:models_weebi/src/models/ticket_weebi_abstract.dart';

import 'package:models_weebi/utils.dart';

mixin TicketSharing on TicketWeebiAbstract {
  // -----STRINGS-----
  String get titleTotalPrice {
    final sb = StringBuffer(numFormat.format(totalPriceTaxAndPromoIncluded))
      ..write(' ');
    sb.write(paiementType.paiementString);
    return sb.toString();
  }

  String get titleSellCovered {
    final sb = StringBuffer(numFormat.format((received)))..write(' ');
    sb.write(paiementType.paiementString);
    return sb.toString();
  }

  String get titleTotalCost {
    final sb = StringBuffer(numFormat.format(totalCostTaxAndPromoIncluded))
      ..write(' ');
    sb.write(paiementType.paiementString);
    return sb.toString();
  }

  String get titleSpendCovered {
    final sb = StringBuffer(numFormat.format((received)))..write(' ');
    sb.write(paiementType.paiementString);
    return sb.toString();
  }

  // ToRuminate - consider splitting this to avoid passing calibres if no basket
  String getSharableText(Iterable<ArticleCalibre> lines) {
    final products = StringBuffer();
    products.write('items : \n');

    for (final item in items) {
      if (ticketType == TicketType.inventory) {
        final temp = item.quantity >= 0
            ? '+${numFormat.format(item.quantity)}'
            : '-${numFormat.format(item.quantity)}';
        products.write(
            '  ${item.article.fullName} = ${numFormat.format(item.inventoryAbsoluteQt)} ($temp)\n');
      } else if (ticketType == TicketType.stockIn) {
        products.write(
            '  ${item.article.fullName} +${numFormat.format(item.quantity)}\n');
      } else if (ticketType == TicketType.stockOut) {
        products.write(
            '  ${item.article.fullName} -${numFormat.format(item.quantity)}\n');
      } else if (ticketType.isPrice) {
        products.write(
            '  ${numFormat.format(item.quantity)}x ${item.article.fullName} ${numFormat.format(item.articlePrice)} = ${numFormat.format(item.totalPrice)}\n');
      } else if (ticketType.isCost) {
        products.write(
            '  ${numFormat.format(item.quantity)}x ${item.article.fullName} ${numFormat.format(item.articleCost)} = ${numFormat.format(item.totalCost)}\n');
      }
    }

    if (ticketType == TicketType.sell) {
      final sb = StringBuffer()
        // ..writeln('#$shopId')
        ..writeln('#ticket : $id')
        ..writeln('date : ${date.toIso8601String()}')
        ..writeln('type: ${ticketType.typeString}')
        ..writeln('paiement : ${paiementType.paiementString}')
        ..writeln(products.toString())
        ..writeln('')
        ..writeln('total : ${numFormat.format(totalPriceItemsOnly)}')
        ..writeln('- ${numFormat.format(totalPricePromoVal)} (promo $promo%)')
        ..writeln(
            'total HT : ${numFormat.format(totalPriceTaxExcludedPromoIncluded)}')
        ..writeln(
            '+ taxe : ${numFormat.format(totalPriceTaxesVal)} (taxe ${taxe.percentage} %)')
        ..writeln(
            'total TTC : ${numFormat.format(totalPriceTaxAndPromoIncluded)}')
        ..writeln(
            'monnaie : ${numFormat.format(received - totalPriceTaxAndPromoIncluded)}')
        ..writeln('note : $comment')
        ..writeln(deactivatedDate)
        ..writeln('')
        ..writeln('contact : $contactId');
      return sb.toString();
    } else if (ticketType == TicketType.spend) {
      final sb = StringBuffer()
        // ..writeln('shopId : $shopId')
        ..writeln('#ticket : $id')
        ..writeln('date : ${date.toIso8601String()}')
        ..writeln('type : ${ticketType.typeString}')
        ..writeln('paiement : ${paiementType.paiementString}')
        ..writeln(products.toString())
        ..writeln('')
        ..writeln('total : ${numFormat.format(totalCostItemsOnly)}')
        ..writeln('- ${numFormat.format(totalCostPromoVal)} (promo $promo %)')
        ..writeln(
            'total HT : ${numFormat.format(totalCostTaxExcludedIncludingPromo)}')
        ..writeln(
            '+ taxe : ${numFormat.format(totalCostTaxesVal)} (taxe ${taxe.percentage} %)')
        ..writeln(
            'total TTC : ${numFormat.format(totalCostTaxAndPromoIncluded)}')
        ..writeln(
            'monnaie : ${numFormat.format(received - totalCostTaxAndPromoIncluded)}')
        ..writeln('note : $comment')
        ..writeln(deactivatedDate)
        ..writeln('')
        ..writeln('contact : $contactId');
      return sb.toString();
    } else if (ticketType == TicketType.sellCovered ||
        ticketType == TicketType.spendCovered) {
      final sb = StringBuffer()
        // ..writeln('shopId : $shopId')
        ..writeln('#ticket : $id')
        ..writeln('date : ${date.toIso8601String()}')
        ..writeln('paiement : ${paiementType.paiementString}')
        ..writeln('type : ${ticketType.typeString}')
        ..writeln('versé : ${numFormat.format(received)}')
        ..writeln('note : $comment')
        ..writeln(deactivatedDate)
        ..writeln('')
        ..writeln('client : $contactId');
      return sb.toString();
    } else if (ticketType == TicketType.sellDeferred) {
      final sb = StringBuffer()
        // ..writeln('shopId : $shopId')
        ..writeln('#ticket : $id')
        ..writeln('date : ${date.toIso8601String()}')
        ..writeln('type : ${ticketType.typeString}')
        ..writeln('paiement : ${paiementType.paiementString}')
        ..writeln(products.toString())
        ..writeln('')
        ..writeln('total : ${numFormat.format(totalPriceItemsOnly)}')
        ..writeln('- ${numFormat.format(totalPricePromoVal)} (promo $promo%)')
        ..writeln(
            'total HT : ${numFormat.format(totalPriceTaxExcludedPromoIncluded)}')
        ..writeln(
            '+ taxe : ${numFormat.format(totalPriceTaxesVal)} (taxe ${taxe.percentage} %)')
        ..writeln(
            'total TTC : ${numFormat.format(totalPriceTaxAndPromoIncluded)}')
        ..writeln('note : $comment')
        ..writeln(deactivatedDate)
        ..writeln('')
        ..writeln('client : $contactId');
      return sb.toString();
    } else if (ticketType == TicketType.spendDeferred) {
      final sb = StringBuffer()
        // ..writeln('shopId : $shopId')
        ..writeln('#ticket : $id')
        ..writeln('date : ${date.toIso8601String()}')
        ..writeln('type : ${ticketType.typeString}')
        ..writeln('paiement : ${paiementType.paiementString}')
        ..writeln(products.toString())
        ..writeln('')
        ..writeln('total : ${numFormat.format(totalCostItemsOnly)}')
        ..writeln('- ${numFormat.format(totalCostPromoVal)} (promo $promo %)')
        ..writeln(
            'total HT : ${numFormat.format(totalCostTaxExcludedIncludingPromo)}')
        ..writeln(
            '+ taxe : ${numFormat.format(totalCostTaxesVal)} (taxe ${taxe.percentage} %)')
        ..writeln(
            'total TTC : ${numFormat.format(totalCostTaxAndPromoIncluded)}')
        ..writeln('note : $comment')
        ..writeln(deactivatedDate)
        ..writeln('')
        ..writeln('fournisseur : $contactId');
      return sb.toString();
    } else {
      final sb = StringBuffer()
        // ..writeln('shopId : $shopId')
        ..writeln('#ticket : $id')
        ..writeln('date : ${date.toIso8601String()}')
        ..writeln('type : ${ticketType.typeString}')
        // ..writeln('paiement: ${paiementType.paiementString}')
        ..writeln(products.toString())
        // ..writeln('contact : $contactId')
        ..writeln('note : $comment')
        ..writeln(deactivatedDate);
      return sb.toString();
    }
  }
}
